@@ -8,20 +8,13 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
-from typing import Union
+from typing import Optional, Union
 
 from hypothesis.internal.compat import int_from_bytes, int_to_bytes
-from hypothesis.internal.conjecture.data import (
-    ConjectureResult,
-    IRType,
-    Status,
-    _Overrun,
-    bits_to_bytes,
-    ir_size_nodes,
-    ir_value_permitted,
-)
-from hypothesis.internal.conjecture.engine import BUFFER_SIZE_IR, ConjectureRunner
-from hypothesis.internal.conjecture.junkdrawer import find_integer
+from hypothesis.internal.conjecture.choice import ChoiceT, choice_permitted
+from hypothesis.internal.conjecture.data import ConjectureResult, Status, _Overrun
+from hypothesis.internal.conjecture.engine import ConjectureRunner
+from hypothesis.internal.conjecture.junkdrawer import bits_to_bytes, find_integer
 from hypothesis.internal.conjecture.pareto import NO_SCORE
 
 
@@ -85,7 +78,7 @@ class Optimiser:
         # We allow transitions that leave the score unchanged as long as they
         # don't increase the number of nodes. This gives us a certain amount of
         # freedom for lateral moves that will take us out of local maxima.
-        if len(data.ir_nodes) <= len(self.current_data.ir_nodes):
+        if len(data.nodes) <= len(self.current_data.nodes):
             self.current_data = data
             return True
         return False
@@ -98,11 +91,11 @@ class Optimiser:
 
         nodes_examined = set()
 
-        prev = None
-        i = len(self.current_data.ir_nodes) - 1
+        prev: Optional[ConjectureResult] = None
+        i = len(self.current_data.nodes) - 1
         while i >= 0 and self.improvements <= self.max_improvements:
             if prev is not self.current_data:
-                i = len(self.current_data.ir_nodes) - 1
+                i = len(self.current_data.nodes) - 1
                 prev = self.current_data
 
             if i in nodes_examined:
@@ -110,19 +103,19 @@ class Optimiser:
                 continue
 
             nodes_examined.add(i)
-            node = self.current_data.ir_nodes[i]
+            node = self.current_data.nodes[i]
             assert node.index is not None
             # we can only (sensibly & easily) define hill climbing for
             # numeric-style nodes. It's not clear hill-climbing a string is
             # useful, for instance.
-            if node.ir_type not in {"integer", "float", "bytes", "boolean"}:
+            if node.type not in {"integer", "float", "bytes", "boolean"}:
                 continue
 
             def attempt_replace(k: int) -> bool:
                 """
                 Try replacing the current node in the current best test case
                 with a value which is "k times larger", where the exact notion
-                of "larger" depends on the ir_type.
+                of "larger" depends on the choice_type.
 
                 Note that we use the *current* best and not the one we started with.
                 This helps ensure that if we luck into a good draw when making
@@ -132,27 +125,27 @@ class Optimiser:
                 if abs(k) > 2**20:
                     return False
 
-                node = self.current_data.ir_nodes[i]
+                node = self.current_data.nodes[i]
                 assert node.index is not None
                 if node.was_forced:
                     return False  # pragma: no cover
 
-                new_value: IRType
-                if node.ir_type in {"integer", "float"}:
+                new_choice: ChoiceT
+                if node.type in {"integer", "float"}:
                     assert isinstance(node.value, (int, float))
-                    new_value = node.value + k
-                elif node.ir_type == "boolean":
+                    new_choice = node.value + k
+                elif node.type == "boolean":
                     assert isinstance(node.value, bool)
                     if abs(k) > 1:
                         return False
                     if k == -1:
-                        new_value = False
+                        new_choice = False
                     if k == 1:
-                        new_value = True
+                        new_choice = True
                     if k == 0:  # pragma: no cover
-                        new_value = node.value
+                        new_choice = node.value
                 else:
-                    assert node.ir_type == "bytes"
+                    assert node.type == "bytes"
                     assert isinstance(node.value, bytes)
                     v = int_from_bytes(node.value)
                     # can't go below zero for bytes
@@ -162,21 +155,20 @@ class Optimiser:
                     # allow adding k to increase the number of bytes. we don't want
                     # to decrease so that b"01" doesn't turn into b"1".
                     size = max(len(node.value), bits_to_bytes(v.bit_length()))
-                    new_value = int_to_bytes(v, size)
+                    new_choice = int_to_bytes(v, size)
 
-                if not ir_value_permitted(new_value, node.ir_type, node.kwargs):
+                if not choice_permitted(new_choice, node.kwargs):
                     return False
 
                 for _ in range(3):
-                    nodes = self.current_data.ir_nodes
-                    attempt_nodes = (
-                        nodes[: node.index]
-                        + (node.copy(with_value=new_value),)
-                        + nodes[node.index + 1 :]
+                    choices = self.current_data.choices
+                    attempt_choices = (
+                        choices[: node.index]
+                        + (new_choice,)
+                        + choices[node.index + 1 :]
                     )
                     attempt = self.engine.cached_test_function_ir(
-                        attempt_nodes,
-                        extend=BUFFER_SIZE_IR - ir_size_nodes(attempt_nodes),
+                        attempt_choices, extend="full"
                     )
 
                     if self.consider_new_data(attempt):
@@ -186,25 +178,23 @@ class Optimiser:
                         return False
 
                     assert isinstance(attempt, ConjectureResult)
-                    if len(attempt.ir_nodes) == len(self.current_data.ir_nodes):
+                    if len(attempt.nodes) == len(self.current_data.nodes):
                         return False
 
                     for j, ex in enumerate(self.current_data.examples):
-                        if ex.ir_start >= node.index + 1:
+                        if ex.start >= node.index + 1:
                             break  # pragma: no cover
-                        if ex.ir_end <= node.index:
+                        if ex.end <= node.index:
                             continue
                         ex_attempt = attempt.examples[j]
-                        if ex.ir_length == ex_attempt.ir_length:
+                        if ex.choice_count == ex_attempt.choice_count:
                             continue  # pragma: no cover
-                        replacement = attempt.ir_nodes[
-                            ex_attempt.ir_start : ex_attempt.ir_end
-                        ]
+                        replacement = attempt.choices[ex_attempt.start : ex_attempt.end]
                         if self.consider_new_data(
                             self.engine.cached_test_function_ir(
-                                nodes[: node.index]
+                                choices[: node.index]
                                 + replacement
-                                + self.current_data.ir_nodes[ex.ir_end :]
+                                + self.current_data.choices[ex.end :]
                             )
                         ):
                             return True
