@@ -11,11 +11,13 @@
 import math
 from collections import defaultdict
 from collections.abc import Sequence
-from typing import TYPE_CHECKING, Callable, Optional, TypeVar, Union
+from typing import TYPE_CHECKING, Callable, Optional, Union, cast
 
 import attr
 
 from hypothesis.internal.conjecture.choice import (
+    ChoiceNode,
+    ChoiceT,
     choice_equal,
     choice_from_index,
     choice_key,
@@ -25,10 +27,10 @@ from hypothesis.internal.conjecture.choice import (
 from hypothesis.internal.conjecture.data import (
     ConjectureData,
     ConjectureResult,
-    IRNode,
+    Examples,
     Status,
+    _Overrun,
     draw_choice,
-    ir_size,
 )
 from hypothesis.internal.conjecture.junkdrawer import (
     endswith,
@@ -52,39 +54,31 @@ from hypothesis.internal.floats import MAX_PRECISE_INTEGER
 
 if TYPE_CHECKING:
     from random import Random
+    from typing import TypeAlias
 
     from hypothesis.internal.conjecture.engine import ConjectureRunner
 
-SortKeyT = TypeVar("SortKeyT", str, bytes)
+ShrinkPredicateT: "TypeAlias" = Callable[[Union[ConjectureResult, _Overrun]], bool]
 
 
-def sort_key(buffer: SortKeyT) -> tuple[int, SortKeyT]:
-    """Returns a sort key such that "simpler" buffers are smaller than
+def sort_key(nodes: Sequence[ChoiceNode]) -> tuple[int, tuple[int, ...]]:
+    """Returns a sort key such that "simpler" choice sequences are smaller than
     "more complicated" ones.
 
     We define sort_key so that x is simpler than y if x is shorter than y or if
-    they have the same length and x < y lexicographically. This is called the
-    shortlex order.
+    they have the same length and map(choice_to_index, x) < map(choice_to_index, y).
 
-    The reason for using the shortlex order is:
+    The reason for using this ordering is:
 
     1. If x is shorter than y then that means we had to make fewer decisions
        in constructing the test case when we ran x than we did when we ran y.
-    2. If x is the same length as y then replacing a byte with a lower byte
-       corresponds to reducing the value of an integer we drew with draw_bits
-       towards zero.
-    3. We want a total order, and given (2) the natural choices for things of
-       the same size are either the lexicographic or colexicographic orders
-       (the latter being the lexicographic order of the reverse of the string).
-       Because values drawn early in generation potentially get used in more
+    2. If x is the same length as y then replacing a choice with a lower index
+       choice corresponds to replacing it with a simpler/smaller choice.
+    3. Because choices drawn early in generation potentially get used in more
        places they potentially have a more significant impact on the final
-       result, so it makes sense to prioritise reducing earlier values over
-       later ones. This makes the lexicographic order the more natural choice.
+       result, so it makes sense to prioritise reducing earlier choices over
+       later ones.
     """
-    return (len(buffer), buffer)  # pragma: no cover # removing soon
-
-
-def sort_key_ir(nodes: Sequence[IRNode]) -> tuple[int, tuple[int, ...]]:
     return (
         len(nodes),
         tuple(choice_to_index(node.value, node.kwargs) for node in nodes),
@@ -113,10 +107,10 @@ class ShrinkPassDefinition:
     run_with_chooser = attr.ib()
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.run_with_chooser.__name__
 
-    def __attrs_post_init__(self):
+    def __attrs_post_init__(self) -> None:
         assert self.name not in SHRINK_PASS_DEFINITIONS, self.name
         SHRINK_PASS_DEFINITIONS[self.name] = self
 
@@ -293,7 +287,7 @@ class Shrinker:
         self,
         engine: "ConjectureRunner",
         initial: Union[ConjectureData, ConjectureResult],
-        predicate: Optional[Callable[[ConjectureData], bool]],
+        predicate: Optional[ShrinkPredicateT],
         *,
         allow_transition: Optional[
             Callable[[Union[ConjectureData, ConjectureResult], ConjectureData], bool]
@@ -337,7 +331,11 @@ class Shrinker:
 
         # Because the shrinker is also used to `pareto_optimise` in the target phase,
         # we sometimes want to allow extending buffers instead of aborting at the end.
-        if in_target_phase:
+        if in_target_phase:  # pragma: no cover
+            # TODO_IR: this is no longer used, but it should be. See
+            # https://github.com/HypothesisWorks/hypothesis/commit/91c63bb76c970effd6cf3c013d8ed98788cf0527
+            # we'll need to adjust for the new notion of size in terms of nodes,
+            # and change self.cached_test_function_ir.
             from hypothesis.internal.conjecture.engine import BUFFER_SIZE
 
             self.__extend = BUFFER_SIZE
@@ -406,41 +404,17 @@ class Shrinker:
         self.check_calls()
         return result
 
-    def consider_new_tree(self, tree: Sequence[IRNode]) -> bool:
-        tree = tree[: len(self.nodes)]
+    def consider_new_nodes(self, nodes: Sequence[ChoiceNode]) -> bool:
+        nodes = nodes[: len(self.nodes)]
 
-        if startswith(tree, self.nodes):
+        if startswith(nodes, self.nodes):
             return True
 
-        if sort_key_ir(self.nodes) < sort_key_ir(tree):
+        if sort_key(self.nodes) < sort_key(nodes):
             return False
 
         previous = self.shrink_target
-        self.cached_test_function_ir(tree)
-        return previous is not self.shrink_target
-
-    def incorporate_new_buffer(
-        self, buffer
-    ):  # pragma: no cover # removing function soon
-        """Either runs the test function on this buffer and returns True if
-        that changed the shrink_target, or determines that doing so would
-        be useless and returns False without running it."""
-
-        buffer = bytes(buffer[: self.shrink_target.index])
-        # Sometimes an attempt at lexicographic minimization will do the wrong
-        # thing because the buffer has changed under it (e.g. something has
-        # turned into a write, the bit size has changed). The result would be
-        # an invalid string, but it's better for us to just ignore it here as
-        # it turns out to involve quite a lot of tricky book-keeping to get
-        # this right and it's better to just handle it in one place.
-        if sort_key(buffer) >= sort_key(self.shrink_target.buffer):
-            return False
-
-        if self.shrink_target.buffer.startswith(buffer):
-            return False
-
-        previous = self.shrink_target
-        self.cached_test_function(buffer)
+        self.cached_test_function_ir(nodes)
         return previous is not self.shrink_target
 
     def incorporate_test_data(self, data):
@@ -450,7 +424,7 @@ class Shrinker:
             return
         if (
             self.__predicate(data)
-            and sort_key_ir(data.ir_nodes) < sort_key_ir(self.shrink_target.ir_nodes)
+            and sort_key(data.nodes) < sort_key(self.shrink_target.nodes)
             and self.__allow_transition(self.shrink_target, data)
         ):
             self.update_shrink_target(data)
@@ -462,7 +436,7 @@ class Shrinker:
     def random(self) -> "Random":
         return self.engine.random
 
-    def shrink(self):
+    def shrink(self) -> None:
         """Run the full set of shrinks and update shrink_target.
 
         This method is "mostly idempotent" - calling it twice is unlikely to
@@ -518,17 +492,16 @@ class Shrinker:
                 self.debug("")
         self.explain()
 
-    def explain(self):
-        from hypothesis.internal.conjecture.engine import BUFFER_SIZE_IR
+    def explain(self) -> None:
 
         if not self.should_explain or not self.shrink_target.arg_slices:
             return
 
-        self.max_stall = 1e999
+        self.max_stall = 2**100
         shrink_target = self.shrink_target
         nodes = self.nodes
         choices = self.choices
-        chunks = defaultdict(list)
+        chunks: dict[tuple[int, int], list[tuple[ChoiceT, ...]]] = defaultdict(list)
 
         # Before we start running experiments, let's check for known inputs which would
         # make them redundant.  The shrinking process means that we've already tried many
@@ -567,30 +540,27 @@ class Shrinker:
                 for i in range(start, end):
                     node = nodes[i]
                     if not node.was_forced:
-                        value = draw_choice(
-                            node.ir_type, node.kwargs, random=self.random
-                        )
+                        value = draw_choice(node.type, node.kwargs, random=self.random)
                         node = node.copy(with_value=value)
                     replacement.append(node.value)
 
                 attempt = choices[:start] + tuple(replacement) + choices[end:]
-                result = self.engine.cached_test_function_ir(
-                    attempt, extend=BUFFER_SIZE_IR - ir_size(attempt)
-                )
+                result = self.engine.cached_test_function_ir(attempt, extend="full")
 
                 # Turns out this was a variable-length part, so grab the infix...
                 if result.status is Status.OVERRUN:
                     continue  # pragma: no cover  # flakily covered
+                result = cast(ConjectureResult, result)
                 if not (
                     len(attempt) == len(result.choices)
-                    and endswith(result.ir_nodes, nodes[end:])
+                    and endswith(result.nodes, nodes[end:])
                 ):
                     for ex, res in zip(shrink_target.examples, result.examples):
-                        assert ex.ir_start == res.ir_start
-                        assert ex.ir_start <= start
+                        assert ex.start == res.start
+                        assert ex.start <= start
                         assert ex.label == res.label
-                        if start == ex.ir_start and end == ex.ir_end:
-                            res_end = res.ir_end
+                        if start == ex.start and end == ex.end:
+                            res_end = res.end
                             break
                     else:
                         raise NotImplementedError("Expected matching prefixes")
@@ -603,6 +573,7 @@ class Shrinker:
 
                     if result.status is Status.OVERRUN:
                         continue  # pragma: no cover  # flakily covered
+                    result = cast(ConjectureResult, result)
                 else:
                     chunks[(start, end)].append(result.choices[start:end])
 
@@ -629,7 +600,7 @@ class Shrinker:
         chunks_by_start_index = sorted(chunks.items())
         for _ in range(500):  # pragma: no branch
             # no-branch here because we don't coverage-test the abort-at-500 logic.
-            new_choices = []
+            new_choices: list[ChoiceT] = []
             prev_end = 0
             for (start, end), ls in chunks_by_start_index:
                 assert prev_end <= start < end, "these chunks must be nonoverlapping"
@@ -654,7 +625,7 @@ class Shrinker:
                     )
                     break
 
-    def greedy_shrink(self):
+    def greedy_shrink(self) -> None:
         """Run a full set of greedy shrinks (that is, ones that will only ever
         move to a better target) and update shrink_target appropriately.
 
@@ -693,11 +664,11 @@ class Shrinker:
 
     @derived_value  # type: ignore
     def examples_starting_at(self):
-        result = [[] for _ in self.shrink_target.ir_nodes]
+        result = [[] for _ in self.shrink_target.nodes]
         for i, ex in enumerate(self.examples):
             # We can have zero-length examples that start at the end
-            if ex.ir_start < len(result):
-                result[ex.ir_start].append(i)
+            if ex.start < len(result):
+                result[ex.start].append(i)
         return tuple(map(tuple, result))
 
     def reduce_each_alternative(self):
@@ -712,11 +683,11 @@ class Shrinker:
         this causes.
         """
         i = 0
-        while i < len(self.shrink_target.ir_nodes):
-            nodes = self.shrink_target.ir_nodes
+        while i < len(self.shrink_target.nodes):
+            nodes = self.shrink_target.nodes
             node = nodes[i]
             if (
-                node.ir_type == "integer"
+                node.type == "integer"
                 and not node.was_forced
                 and node.value <= 10
                 and node.kwargs["min_value"] == 0
@@ -741,14 +712,14 @@ class Shrinker:
                     and zero_attempt is not None
                     and zero_attempt.status >= Status.VALID
                 ):
-                    changed_shape = len(zero_attempt.ir_nodes) != len(nodes)
+                    changed_shape = len(zero_attempt.nodes) != len(nodes)
 
                     if not changed_shape:
                         for j in range(i + 1, len(nodes)):
-                            zero_node = zero_attempt.ir_nodes[j]
+                            zero_node = zero_attempt.nodes[j]
                             orig_node = nodes[j]
                             if (
-                                zero_node.ir_type != orig_node.ir_type
+                                zero_node.type != orig_node.type
                                 or not choice_permitted(
                                     orig_node.value, zero_node.kwargs
                                 )
@@ -762,10 +733,10 @@ class Shrinker:
             i += 1
 
     def try_lower_node_as_alternative(self, i, v):
-        """Attempt to lower `self.shrink_target.ir_nodes[i]` to `v`,
+        """Attempt to lower `self.shrink_target.nodes[i]` to `v`,
         while rerandomising and attempting to repair any subsequent
         changes to the shape of the test case that this causes."""
-        nodes = self.shrink_target.ir_nodes
+        nodes = self.shrink_target.nodes
         initial_attempt = self.cached_test_function_ir(
             nodes[:i] + (nodes[i].copy(with_value=v),) + nodes[i + 1 :]
         )
@@ -777,7 +748,7 @@ class Shrinker:
         examples = self.examples_starting_at[i]
         for _ in range(3):
             random_attempt = self.engine.cached_test_function_ir(
-                [n.value for n in prefix], extend=len(nodes) * 2
+                [n.value for n in prefix], extend=len(nodes)
             )
             if random_attempt.status < Status.VALID:
                 continue
@@ -785,12 +756,8 @@ class Shrinker:
             for j in examples:
                 initial_ex = initial.examples[j]
                 attempt_ex = random_attempt.examples[j]
-                contents = random_attempt.ir_nodes[
-                    attempt_ex.ir_start : attempt_ex.ir_end
-                ]
-                self.consider_new_tree(
-                    nodes[:i] + contents + nodes[initial_ex.ir_end :]
-                )
+                contents = random_attempt.nodes[attempt_ex.start : attempt_ex.end]
+                self.consider_new_nodes(nodes[:i] + contents + nodes[initial_ex.end :])
                 if initial is not self.shrink_target:
                     return True
         return False
@@ -896,15 +863,15 @@ class Shrinker:
             passes.sort(key=reordering.__getitem__)
 
     @property
-    def nodes(self):
-        return self.shrink_target.ir_nodes
+    def nodes(self) -> tuple[ChoiceNode, ...]:
+        return self.shrink_target.nodes
 
     @property
-    def choices(self):
+    def choices(self) -> tuple[ChoiceT, ...]:
         return self.shrink_target.choices
 
     @property
-    def examples(self):
+    def examples(self) -> Examples:
         return self.shrink_target.examples
 
     @derived_value  # type: ignore
@@ -949,7 +916,7 @@ class Shrinker:
         i = chooser.choose(range(len(ls) - 1))
         ancestor = ls[i]
 
-        if i + 1 == len(ls) or ls[i + 1].ir_start >= ancestor.ir_end:
+        if i + 1 == len(ls) or ls[i + 1].start >= ancestor.end:
             return
 
         @self.cached(label, i)
@@ -958,22 +925,22 @@ class Shrinker:
             hi = len(ls)
             while lo + 1 < hi:
                 mid = (lo + hi) // 2
-                if ls[mid].ir_start >= ancestor.ir_end:
+                if ls[mid].start >= ancestor.end:
                     hi = mid
                 else:
                     lo = mid
-            return [t for t in ls[i + 1 : hi] if t.ir_length < ancestor.ir_length]
+            return [t for t in ls[i + 1 : hi] if t.choice_count < ancestor.choice_count]
 
-        descendant = chooser.choose(descendants, lambda ex: ex.ir_length > 0)
+        descendant = chooser.choose(descendants, lambda ex: ex.choice_count > 0)
 
-        assert ancestor.ir_start <= descendant.ir_start
-        assert ancestor.ir_end >= descendant.ir_end
-        assert descendant.ir_length < ancestor.ir_length
+        assert ancestor.start <= descendant.start
+        assert ancestor.end >= descendant.end
+        assert descendant.choice_count < ancestor.choice_count
 
-        self.consider_new_tree(
-            self.nodes[: ancestor.ir_start]
-            + self.nodes[descendant.ir_start : descendant.ir_end]
-            + self.nodes[ancestor.ir_end :]
+        self.consider_new_nodes(
+            self.nodes[: ancestor.start]
+            + self.nodes[descendant.start : descendant.end]
+            + self.nodes[ancestor.end :]
         )
 
     def lower_common_node_offset(self):
@@ -1015,7 +982,7 @@ class Shrinker:
         changed = []
         for i in sorted(self.__changed_nodes):
             node = self.nodes[i]
-            if node.trivial or node.ir_type != "integer":
+            if node.trivial or node.type != "integer":
                 continue
             changed.append(node)
 
@@ -1039,9 +1006,9 @@ class Shrinker:
             )
 
         def consider(n, sign):
-            return self.consider_new_tree(
+            return self.consider_new_nodes(
                 replace_all(
-                    st.ir_nodes,
+                    st.nodes,
                     [
                         offset_node(node, sign * (n + v))
                         for node, v in zip(changed, ints)
@@ -1062,26 +1029,26 @@ class Shrinker:
         self.__changed_nodes.add(i)
 
     @property
-    def __changed_nodes(self):
+    def __changed_nodes(self) -> set[int]:
         if self.__last_checked_changed_at is self.shrink_target:
             return self.__all_changed_nodes
 
         prev_target = self.__last_checked_changed_at
         new_target = self.shrink_target
         assert prev_target is not new_target
-        prev_nodes = prev_target.ir_nodes
-        new_nodes = new_target.ir_nodes
-        assert sort_key_ir(new_target.ir_nodes) < sort_key_ir(prev_target.ir_nodes)
+        prev_nodes = prev_target.nodes
+        new_nodes = new_target.nodes
+        assert sort_key(new_target.nodes) < sort_key(prev_target.nodes)
 
         if len(prev_nodes) != len(new_nodes) or any(
-            n1.ir_type != n2.ir_type for n1, n2 in zip(prev_nodes, new_nodes)
+            n1.type != n2.type for n1, n2 in zip(prev_nodes, new_nodes)
         ):
             # should we check kwargs are equal as well?
             self.__all_changed_nodes = set()
         else:
             assert len(prev_nodes) == len(new_nodes)
             for i, (n1, n2) in enumerate(zip(prev_nodes, new_nodes)):
-                assert n1.ir_type == n2.ir_type
+                assert n1.type == n2.type
                 if not choice_equal(n1.value, n2.value):
                     self.__all_changed_nodes.add(i)
 
@@ -1170,16 +1137,16 @@ class Shrinker:
             # case of this function of preserving from the right instead of
             # preserving from the left. see test_can_shrink_variable_string_draws.
 
-            (index, attempt_ir_type, attempt_kwargs, _attempt_forced) = (
+            (index, attempt_choice_type, attempt_kwargs, _attempt_forced) = (
                 attempt.misaligned_at
             )
             node = self.nodes[index]
-            if node.ir_type != attempt_ir_type:
+            if node.type != attempt_choice_type:
                 return False  # pragma: no cover
             if node.was_forced:
                 return False  # pragma: no cover
 
-            if node.ir_type in {"string", "bytes"}:
+            if node.type in {"string", "bytes"}:
                 # if the size *increased*, we would have to guess what to pad with
                 # in order to try fixing up this attempt. Just give up.
                 if node.kwargs["min_size"] <= attempt_kwargs["min_size"]:
@@ -1189,7 +1156,7 @@ class Shrinker:
                 # the size decreased in our attempt. Try again, but replace with
                 # the min_size that we would have gotten, and truncate the value
                 # to that size by removing any elements past min_size.
-                return self.consider_new_tree(
+                return self.consider_new_nodes(
                     initial_attempt[: node.index]
                     + [
                         initial_attempt[node.index].copy(
@@ -1202,7 +1169,7 @@ class Shrinker:
                     + initial_attempt[node.index :]
                 )
 
-        lost_nodes = len(self.nodes) - len(attempt.ir_nodes)
+        lost_nodes = len(self.nodes) - len(attempt.nodes)
         if lost_nodes <= 0:
             return False
 
@@ -1216,17 +1183,17 @@ class Shrinker:
         regions_to_delete = {(end, end + lost_nodes)}
 
         for ex in self.examples:
-            if ex.ir_start > start:
+            if ex.start > start:
                 continue
-            if ex.ir_end <= end:
+            if ex.end <= end:
                 continue
 
             if ex.index >= len(attempt.examples):
                 continue  # pragma: no cover
 
             replacement = attempt.examples[ex.index]
-            in_original = [c for c in ex.children if c.ir_start >= end]
-            in_replaced = [c for c in replacement.children if c.ir_start >= end]
+            in_original = [c for c in ex.children if c.start >= end]
+            in_replaced = [c for c in replacement.children if c.start >= end]
 
             if len(in_replaced) >= len(in_original) or not in_replaced:
                 continue
@@ -1238,12 +1205,12 @@ class Shrinker:
             # important, so we try to arrange it so that it retains its
             # rightmost children instead of its leftmost.
             regions_to_delete.add(
-                (in_original[0].ir_start, in_original[-len(in_replaced)].ir_start)
+                (in_original[0].start, in_original[-len(in_replaced)].start)
             )
 
         for u, v in sorted(regions_to_delete, key=lambda x: x[1] - x[0], reverse=True):
             try_with_deleted = initial_attempt[:u] + initial_attempt[v:]
-            if self.consider_new_tree(try_with_deleted):
+            if self.consider_new_nodes(try_with_deleted):
                 return True
 
         return False
@@ -1269,11 +1236,11 @@ class Shrinker:
 
             for ex in self.shrink_target.examples:
                 if (
-                    ex.ir_length > 0
+                    ex.choice_count > 0
                     and ex.discarded
-                    and (not discarded or ex.ir_start >= discarded[-1][-1])
+                    and (not discarded or ex.start >= discarded[-1][-1])
                 ):
-                    discarded.append((ex.ir_start, ex.ir_end))
+                    discarded.append((ex.start, ex.end))
 
             # This can happen if we have discards but they are all of
             # zero length. This shouldn't happen very often so it's
@@ -1286,16 +1253,16 @@ class Shrinker:
             for u, v in reversed(discarded):
                 del attempt[u:v]
 
-            if not self.consider_new_tree(tuple(attempt)):
+            if not self.consider_new_nodes(tuple(attempt)):
                 return False
         return True
 
     @derived_value  # type: ignore
     def duplicated_nodes(self):
-        """Returns a list of nodes grouped (ir_type, value)."""
+        """Returns a list of nodes grouped (choice_type, value)."""
         duplicates = defaultdict(list)
         for node in self.nodes:
-            duplicates[(node.ir_type, choice_key(node.value))].append(node)
+            duplicates[(node.type, choice_key(node.value))].append(node)
         return list(duplicates.values())
 
     @defines_shrink_pass()
@@ -1344,8 +1311,8 @@ class Shrinker:
             # The motivation for the last condition is to avoid trying weird
             # non-shrinks where we raise one node and think we lowered another
             # (but didn't).
-            return node.ir_type in {"integer", "float"} and not (
-                node.ir_type == "float"
+            return node.type in {"integer", "float"} and not (
+                node.type == "float"
                 and (math.isnan(node.value) or abs(node.value) >= MAX_PRECISE_INTEGER)
             )
 
@@ -1381,10 +1348,10 @@ class Shrinker:
 
             # if we've increased node2 to the point that we're past max precision,
             # give up - things have become too unstable.
-            if node1.ir_type == "float" and v2 >= MAX_PRECISE_INTEGER:
+            if node1.type == "float" and v2 >= MAX_PRECISE_INTEGER:
                 return False
 
-            return self.consider_new_tree(
+            return self.consider_new_nodes(
                 self.nodes[: node1.index]
                 + (node1.copy(with_value=v1),)
                 + self.nodes[node1.index + 1 : node2.index]
@@ -1397,13 +1364,14 @@ class Shrinker:
     @defines_shrink_pass()
     def lower_integers_together(self, chooser):
         node1 = chooser.choose(
-            self.nodes, lambda n: n.ir_type == "integer" and not n.trivial
+            self.nodes, lambda n: n.type == "integer" and not n.trivial
         )
         # Search up to 3 nodes ahead, to avoid quadratic time.
         node2 = self.nodes[
             chooser.choose(
                 range(node1.index + 1, min(len(self.nodes), node1.index + 3 + 1)),
-                lambda i: self.nodes[i].ir_type == "integer",
+                lambda i: self.nodes[i].type == "integer"
+                and not self.nodes[i].was_forced,
             )
         ]
 
@@ -1415,7 +1383,7 @@ class Shrinker:
         shrink_towards = node1.kwargs["shrink_towards"]
 
         def consider(n):
-            return self.consider_new_tree(
+            return self.consider_new_nodes(
                 self.nodes[: node1.index]
                 + (node1.copy(with_value=node1.value - n),)
                 + self.nodes[node1.index + 1 : node2.index]
@@ -1427,9 +1395,9 @@ class Shrinker:
         find_integer(lambda n: consider(n - shrink_towards))
 
     def minimize_nodes(self, nodes):
-        ir_type = nodes[0].ir_type
+        choice_type = nodes[0].type
         value = nodes[0].value
-        # unlike ir_type and value, kwargs are *not* guaranteed to be equal among all
+        # unlike choice_type and value, kwargs are *not* guaranteed to be equal among all
         # passed nodes. We arbitrarily use the kwargs of the first node. I think
         # this is unsound (= leads to us trying shrinks that could not have been
         # generated), but those get discarded at test-time, and this enables useful
@@ -1437,11 +1405,11 @@ class Shrinker:
         # same operation on both basically just works.
         kwargs = nodes[0].kwargs
         assert all(
-            node.ir_type == ir_type and choice_equal(node.value, value)
+            node.type == choice_type and choice_equal(node.value, value)
             for node in nodes
         )
 
-        if ir_type == "integer":
+        if choice_type == "integer":
             shrink_towards = kwargs["shrink_towards"]
             # try shrinking from both sides towards shrink_towards.
             # we're starting from n = abs(shrink_towards - value). Because the
@@ -1456,7 +1424,7 @@ class Shrinker:
                 abs(shrink_towards - value),
                 lambda n: self.try_shrinking_nodes(nodes, shrink_towards - n),
             )
-        elif ir_type == "float":
+        elif choice_type == "float":
             self.try_shrinking_nodes(nodes, abs(value))
             Float.shrink(
                 abs(value),
@@ -1466,18 +1434,18 @@ class Shrinker:
                 abs(value),
                 lambda val: self.try_shrinking_nodes(nodes, -val),
             )
-        elif ir_type == "boolean":
+        elif choice_type == "boolean":
             # must be True, otherwise would be trivial and not selected.
             assert value is True
             # only one thing to try: false!
             self.try_shrinking_nodes(nodes, False)
-        elif ir_type == "bytes":
+        elif choice_type == "bytes":
             Bytes.shrink(
                 value,
                 lambda val: self.try_shrinking_nodes(nodes, val),
                 min_size=kwargs["min_size"],
             )
-        elif ir_type == "string":
+        elif choice_type == "string":
             String.shrink(
                 value,
                 lambda val: self.try_shrinking_nodes(nodes, val),
@@ -1492,22 +1460,22 @@ class Shrinker:
         i = chooser.choose(range(len(self.examples)))
 
         prev = self.shrink_target
-        nodes = self.shrink_target.ir_nodes
+        nodes = self.shrink_target.nodes
         ex = self.examples[i]
-        prefix = nodes[: ex.ir_start]
+        prefix = nodes[: ex.start]
         replacement = tuple(
             [
                 (
                     node
                     if node.was_forced
                     else node.copy(
-                        with_value=choice_from_index(0, node.ir_type, node.kwargs)
+                        with_value=choice_from_index(0, node.type, node.kwargs)
                     )
                 )
-                for node in nodes[ex.ir_start : ex.ir_end]
+                for node in nodes[ex.start : ex.end]
             ]
         )
-        suffix = nodes[ex.ir_end :]
+        suffix = nodes[ex.end :]
         attempt = self.cached_test_function_ir(prefix + replacement + suffix)
 
         if self.shrink_target is not prev:
@@ -1515,8 +1483,8 @@ class Shrinker:
 
         if isinstance(attempt, ConjectureResult):
             new_ex = attempt.examples[i]
-            new_replacement = attempt.ir_nodes[new_ex.ir_start : new_ex.ir_end]
-            self.consider_new_tree(prefix + new_replacement + suffix)
+            new_replacement = attempt.nodes[new_ex.start : new_ex.end]
+            self.consider_new_nodes(prefix + new_replacement + suffix)
 
     @defines_shrink_pass()
     def minimize_individual_nodes(self, chooser):
@@ -1557,7 +1525,7 @@ class Shrinker:
         # the size of the generated input, we'll try deleting things after that
         # node and see if the resulting attempt works.
 
-        if node.ir_type != "integer":
+        if node.type != "integer":
             # Only try this fixup logic on integer draws. Almost all size
             # dependencies are on integer draws, and if it's not, it's doing
             # something convoluted enough that it is unlikely to shrink well anyway.
@@ -1576,8 +1544,8 @@ class Shrinker:
         if (
             attempt is None
             or attempt.status < Status.VALID
-            or len(attempt.ir_nodes) == len(self.nodes)
-            or len(attempt.ir_nodes) == node.index + 1
+            or len(attempt.nodes) == len(self.nodes)
+            or len(attempt.nodes) == node.index + 1
         ):
             # no point in trying our size-dependency-logic if our attempt at
             # lowering the node resulted in:
@@ -1597,7 +1565,7 @@ class Shrinker:
             while lo + 1 < hi:
                 mid = (lo + hi) // 2
                 ex = self.examples[mid]
-                if ex.ir_start >= node.index:
+                if ex.start >= node.index:
                     hi = mid
                 else:
                     lo = mid
@@ -1611,13 +1579,13 @@ class Shrinker:
             ex = self.examples[
                 chooser.choose(
                     range(first_example_after_node, len(self.examples)),
-                    lambda i: self.examples[i].ir_length > 0,
+                    lambda i: self.examples[i].choice_count > 0,
                 )
             ]
-            self.consider_new_tree(lowered[: ex.ir_start] + lowered[ex.ir_end :])
+            self.consider_new_nodes(lowered[: ex.start] + lowered[ex.end :])
         else:
             node = self.nodes[chooser.choose(range(node.index + 1, len(self.nodes)))]
-            self.consider_new_tree(lowered[: node.index] + lowered[node.index + 1 :])
+            self.consider_new_nodes(lowered[: node.index] + lowered[node.index + 1 :])
 
     @defines_shrink_pass()
     def reorder_examples(self, chooser):
@@ -1646,26 +1614,24 @@ class Shrinker:
         if len(examples) <= 1:
             return
         st = self.shrink_target
-        endpoints = [(ex.ir_start, ex.ir_end) for ex in examples]
+        endpoints = [(ex.start, ex.end) for ex in examples]
 
         Ordering.shrink(
             range(len(examples)),
-            lambda indices: self.consider_new_tree(
+            lambda indices: self.consider_new_nodes(
                 replace_all(
-                    st.ir_nodes,
+                    st.nodes,
                     [
                         (
                             u,
                             v,
-                            st.ir_nodes[examples[i].ir_start : examples[i].ir_end],
+                            st.nodes[examples[i].start : examples[i].end],
                         )
                         for (u, v), i in zip(endpoints, indices)
                     ],
                 )
             ),
-            key=lambda i: sort_key_ir(
-                st.ir_nodes[examples[i].ir_start : examples[i].ir_end]
-            ),
+            key=lambda i: sort_key(st.nodes[examples[i].start : examples[i].end]),
         )
 
     def run_node_program(self, i, description, original, repeats=1):
@@ -1683,9 +1649,9 @@ class Shrinker:
         Returns True if this successfully changes the underlying shrink target,
         else False.
         """
-        if i + len(description) > len(original.ir_nodes) or i < 0:
+        if i + len(description) > len(original.nodes) or i < 0:
             return False
-        attempt = list(original.ir_nodes)
+        attempt = list(original.nodes)
         for _ in range(repeats):
             for k, command in reversed(list(enumerate(description))):
                 j = i + k
@@ -1697,7 +1663,7 @@ class Shrinker:
                 else:
                     raise NotImplementedError(f"Unrecognised command {command!r}")
 
-        return self.consider_new_tree(attempt)
+        return self.consider_new_nodes(attempt)
 
 
 def shrink_pass_family(f):
