@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 from random import Random
 
@@ -22,14 +17,15 @@ def test_find_uses_provided_random():
     prev = None
 
     for _ in range(3):
-        seen = []
+        seen = None
 
         def test(v):
             if len(v) > 5:
-                if seen:
-                    return v == seen[0]
+                nonlocal seen
+                if seen is not None:
+                    return v == seen
                 else:
-                    seen.append(v)
+                    seen = v
                     return True
 
         result = find(
