@@ -15,7 +15,7 @@ from copy import copy
 
 import pytest
 
-from hypothesis import assume, given, strategies as st
+from hypothesis import HealthCheck, assume, given, settings, strategies as st
 from hypothesis.internal.compat import ExceptionGroup
 from hypothesis.strategies._internal.random import (
     RANDOM_METHODS,
@@ -294,6 +294,7 @@ def test_invalid_sample():
 
 
 def test_triangular_modes():
+    @settings(report_multiple_bugs=True)
     @given(st.randoms(use_true_random=False))
     def test(rnd):
         x = rnd.triangular(0.0, 1.0, mode=0.5)
@@ -323,10 +324,6 @@ def test_choices_have_right_length(rnd, choices):
     assert len(rnd.choices(seq, k=k)) == k
 
 
-@pytest.mark.skipif(
-    "randbytes" not in RANDOM_METHODS,
-    reason="randbytes not supported on this Python version",
-)
 @given(any_random, st.integers(0, 100))
 def test_randbytes_have_right_length(rnd, n):
     assert len(rnd.randbytes(n)) == n
@@ -341,6 +338,7 @@ def test_can_manage_very_long_ranges_with_step(rnd):
     assert i in range(0, 2**256, 3)
 
 
+@settings(suppress_health_check=[HealthCheck.too_slow])
 @given(any_random, st.data())
 def test_range_with_arbitrary_step_is_in_range(rnd, data):
     endpoints = st.integers(-100, 100)
