@@ -29,11 +29,11 @@ from hypothesis.errors import DidNotReproduce, InvalidArgument, UnsatisfiedAssum
 from hypothesis.internal.conjecture.choice import choice_equal
 
 from tests.common.utils import capture_out, no_shrink
-from tests.conjecture.common import ir, ir_nodes
+from tests.conjecture.common import ir, nodes
 
 
 @example(ir("0" * 100))  # shorter compressed than not
-@given(st.lists(ir_nodes()))
+@given(st.lists(nodes()))
 def test_encoding_loop(nodes):
     choices = [n.value for n in nodes]
     looped = decode_failure(encode_failure(choices))
@@ -45,7 +45,7 @@ def test_encoding_loop(nodes):
 @example(base64.b64encode(b"\2\3\4"))
 @example(b"\t")
 @example(base64.b64encode(b"\1\0"))  # zlib error
-@example(base64.b64encode(b"\1" + zlib.compress(b"\xFF")))  # ir_from_bytes error
+@example(base64.b64encode(b"\1" + zlib.compress(b"\xff")))  # choices_from_bytes error
 @given(st.binary())
 def test_decoding_may_fail(t):
     try:
@@ -103,9 +103,9 @@ def test_errors_with_did_not_reproduce_if_the_shape_changes():
     n = len(b)
 
     @reproduce_failure(__version__, encode_failure([b]))
-    @given(st.binary(min_size=n + 1, max_size=n + 1))
-    def test(x):
-        assert x == b
+    @given(st.binary(min_size=n, max_size=n) | st.integers())
+    def test(v):
+        assert v == b
 
     with pytest.raises(DidNotReproduce):
         test()
