@@ -11,20 +11,19 @@
 import abc
 import enum
 import sys
+import typing
+from collections.abc import Sequence
 from inspect import Parameter as P, Signature
-from typing import Callable, Dict, Generic, List, Sequence, TypeVar, Union
+from typing import Callable, Generic, List as _List, TypeVar, Union
 
 import pytest
 
 from hypothesis import given, infer, settings, strategies as st
-from hypothesis.errors import (
-    HypothesisDeprecationWarning,
-    InvalidArgument,
-    ResolutionFailed,
-)
+from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.internal.compat import get_type_hints
 from hypothesis.internal.reflection import get_pretty_function_description
 from hypothesis.strategies._internal import types
+from hypothesis.strategies._internal.lazy import LazyStrategy
 from hypothesis.strategies._internal.types import _global_type_lookup
 from hypothesis.strategies._internal.utils import _strategies
 
@@ -36,42 +35,11 @@ from tests.common.debug import (
 )
 from tests.common.utils import fails_with, temp_registered
 
-# Build a set of all types output by core strategies
-blocklist = {
-    "builds",
-    "data",
-    "deferred",
-    "from_regex",
-    "from_type",
-    "ip_addresses",
-    "iterables",
-    "just",
-    "nothing",
-    "one_of",
-    "permutations",
-    "random_module",
-    "randoms",
-    "recursive",
-    "runner",
-    "sampled_from",
-    "shared",
-    "timezone_keys",
-    "timezones",
+types_with_core_strat = {
+    type_
+    for type_, strat in _global_type_lookup.items()
+    if isinstance(strat, LazyStrategy) and strat.function in vars(st).values()
 }
-assert set(_strategies).issuperset(blocklist), blocklist.difference(_strategies)
-types_with_core_strat = set()
-for thing in (
-    getattr(st, name)
-    for name in sorted(_strategies)
-    if name in dir(st) and name not in blocklist
-):
-    for n in range(3):
-        try:
-            ex = find_any(thing(*([st.nothing()] * n)))
-            types_with_core_strat.add(type(ex))
-            break
-        except (TypeError, InvalidArgument, HypothesisDeprecationWarning):
-            continue
 
 
 @pytest.mark.skipif(sys.version_info[:2] >= (3, 14), reason="FIXME-py314")
@@ -91,7 +59,44 @@ def test_resolve_core_strategies(typ):
 
 
 def test_lookup_knows_about_all_core_strategies():
-    cannot_lookup = types_with_core_strat - set(types._global_type_lookup)
+    # Build a set of all types output by core strategies
+    blocklist = {
+        "builds",
+        "data",
+        "deferred",
+        "from_regex",
+        "from_type",
+        "ip_addresses",
+        "iterables",
+        "just",
+        "nothing",
+        "one_of",
+        "permutations",
+        "random_module",
+        "randoms",
+        "recursive",
+        "runner",
+        "sampled_from",
+        "shared",
+        "timezone_keys",
+        "timezones",
+    }
+    assert set(_strategies).issuperset(blocklist), blocklist.difference(_strategies)
+    found = set()
+    for thing in (
+        getattr(st, name)
+        for name in sorted(_strategies)
+        if name in dir(st) and name not in blocklist
+    ):
+        for n in range(3):
+            try:
+                ex = find_any(thing(*([st.nothing()] * n)))
+                found.add(type(ex))
+                break
+            except Exception:
+                continue
+
+    cannot_lookup = found - set(types._global_type_lookup)
     assert not cannot_lookup
 
 
@@ -247,6 +252,7 @@ def _check_instances(t):
         t.__module__ != "typing"
         and t.__name__ != "ByteString"
         and not t.__module__.startswith("pydantic")
+        and t.__module__ != "typing_extensions"
     )
 
 
@@ -275,7 +281,7 @@ class Lines(Sequence[str]):
     """
 
 
-class SpecificDict(Dict[int, int]):
+class SpecificDict(dict[int, int]):
     pass
 
 
@@ -304,10 +310,10 @@ def test_issue_2951_regression():
 
 def test_issue_2951_regression_two_params():
     map_strat = st.builds(SpecificDict, st.dictionaries(st.integers(), st.integers()))
-    expected = repr(st.from_type(Dict[int, int]))
+    expected = repr(st.from_type(dict[int, int]))
     with temp_registered(SpecificDict, map_strat):
         assert st.from_type(SpecificDict) == map_strat
-        assert expected == repr(st.from_type(Dict[int, int]))
+        assert expected == repr(st.from_type(dict[int, int]))
 
 
 @pytest.mark.parametrize(
@@ -328,15 +334,20 @@ def test_generic_origin_with_type_args(generic, strategy):
     assert generic not in types._global_type_lookup
 
 
+skip_39 = pytest.mark.skipif(sys.version_info[:2] == (3, 9), reason="early version")
+
+
 @pytest.mark.parametrize(
     "generic",
     (
         Callable,
-        List,
+        list,
         Sequence,
         # you can register types with all generic parameters
-        List[T],
-        Sequence[T],
+        _List[T],
+        getattr(typing, "Sequence", None)[T],  # pyupgrade workaround
+        pytest.param(list[T], marks=skip_39),
+        pytest.param(Sequence[T], marks=skip_39),
         # User-defined generics should also work
         MyGeneric,
         MyGeneric[T],
