@@ -1,21 +1,17 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 from pytest import raises
 
 from hypothesis import Verbosity, given, settings, strategies as st
+
 from tests.common.utils import no_shrink
 
 
@@ -29,7 +25,7 @@ def test_nesting_1():
         )
         def test_nest(y):
             if y >= x:
-                raise ValueError()
+                raise ValueError
 
         with raises(ValueError):
             test_nest()
