@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import pytest
 
@@ -26,7 +21,7 @@ def test_note_deprecation_blames_right_code_issue_652():
     @st.composite
     def deprecated_strategy(draw):
         draw(st.none())
-        note_deprecation(msg, since="RELEASEDAY")
+        note_deprecation(msg, since="RELEASEDAY", has_codemod=False)
 
     @given(deprecated_strategy())
     def f(x):
@@ -49,3 +44,20 @@ def test_note_deprecation_blames_right_code_issue_652():
 )
 def test_performance_issue_2027(x, y):
     pass
+
+
+@given(
+    st.lists(
+        st.floats(allow_infinity=False),
+        unique=True,
+    )
+)
+def test_unique_floats_with_nan_is_not_flaky_3926(ls):
+    pass
+
+
+# this will take a while to find the regression, but will eventually trigger it.
+# min_value=0 is critical to trigger the probing behavior which exhausts our buffer.
+@given(st.integers(min_value=0, max_value=1 << 25_000))
+def test_overrun_during_datatree_simulation_3874(n):
+    pass
