@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import math
 import sys
@@ -20,6 +15,7 @@ import pytest
 
 from hypothesis import given
 from hypothesis.strategies import floats, integers, lists
+
 from tests.common.debug import minimal
 
 
@@ -36,10 +32,10 @@ def test_positive_negative_int():
 boundaries = pytest.mark.parametrize(
     "boundary",
     sorted(
-        [2 ** i for i in range(10)]
-        + [2 ** i - 1 for i in range(10)]
-        + [2 ** i + 1 for i in range(10)]
-        + [10 ** i for i in range(6)]
+        [2**i for i in range(10)]
+        + [2**i - 1 for i in range(10)]
+        + [2**i + 1 for i in range(10)]
+        + [10**i for i in range(6)]
     ),
 )
 
@@ -62,7 +58,7 @@ def test_minimizes_ints_from_down_to_boundary(boundary):
 
     assert minimal(integers(min_value=boundary - 10), is_good) == boundary
 
-    assert minimal(integers(min_value=boundary), lambda x: True) == boundary
+    assert minimal(integers(min_value=boundary)) == boundary
 
 
 def test_minimizes_negative_integer_range_upwards():
@@ -71,20 +67,20 @@ def test_minimizes_negative_integer_range_upwards():
 
 @boundaries
 def test_minimizes_integer_range_to_boundary(boundary):
-    assert minimal(integers(boundary, boundary + 100), lambda x: True) == boundary
+    assert minimal(integers(boundary, boundary + 100)) == boundary
 
 
 def test_single_integer_range_is_range():
-    assert minimal(integers(1, 1), lambda x: True) == 1
+    assert minimal(integers(1, 1)) == 1
 
 
 def test_minimal_small_number_in_large_range():
-    assert minimal(integers((-(2 ** 32)), 2 ** 32), lambda x: x >= 101) == 101
+    assert minimal(integers((-(2**32)), 2**32), lambda x: x >= 101) == 101
 
 
 def test_minimal_small_sum_float_list():
-    xs = minimal(lists(floats(), min_size=10), lambda x: sum(x) >= 1.0)
-    assert sum(xs) <= 2.0
+    xs = minimal(lists(floats(), min_size=5), lambda x: sum(x) >= 1.0)
+    assert xs == [0.0, 0.0, 0.0, 0.0, 1.0]
 
 
 def test_minimals_boundary_floats():
@@ -92,16 +88,20 @@ def test_minimals_boundary_floats():
         print(x)
         return True
 
-    assert -1 <= minimal(floats(min_value=-1, max_value=1), f) <= 1
+    assert minimal(floats(min_value=-1, max_value=1), f) == 0
 
 
 def test_minimal_non_boundary_float():
     x = minimal(floats(min_value=1, max_value=9), lambda x: x > 2)
-    assert 2 < x < 3
+    assert x == 3  # (the smallest integer > 2)
 
 
 def test_minimal_float_is_zero():
-    assert minimal(floats(), lambda x: True) == 0.0
+    assert minimal(floats()) == 0.0
+
+
+def test_minimal_asymetric_bounded_float():
+    assert minimal(floats(min_value=1.1, max_value=1.6)) == 1.5
 
 
 def test_negative_floats_simplify_to_zero():
@@ -126,7 +126,7 @@ def test_minimize_nan():
 
 def test_minimize_very_large_float():
     t = sys.float_info.max / 2
-    assert t <= minimal(floats(), lambda x: x >= t) < math.inf
+    assert minimal(floats(), lambda x: x >= t) == t
 
 
 def is_integral(value):
@@ -137,7 +137,7 @@ def is_integral(value):
 
 
 def test_can_minimal_float_far_from_integral():
-    minimal(floats(), lambda x: math.isfinite(x) and not is_integral(x * (2 ** 32)))
+    minimal(floats(), lambda x: math.isfinite(x) and not is_integral(x * (2**32)))
 
 
 def test_list_of_fractional_float():
@@ -145,20 +145,18 @@ def test_list_of_fractional_float():
         minimal(
             lists(floats(), min_size=5),
             lambda x: len([t for t in x if t >= 1.5]) >= 5,
-            timeout_after=60,
         )
-    ).issubset([1.5, 2.0])
+    ) == {2}
 
 
 def test_minimal_fractional_float():
-    assert minimal(floats(), lambda x: x >= 1.5) in (1.5, 2.0)
+    assert minimal(floats(), lambda x: x >= 1.5) == 2
 
 
 def test_minimizes_lists_of_negative_ints_up_to_boundary():
     result = minimal(
         lists(integers(), min_size=10),
         lambda x: len([t for t in x if t <= -1]) >= 10,
-        timeout_after=60,
     )
     assert result == [-1] * 10
 
@@ -176,13 +174,13 @@ def test_floats_in_constrained_range(left, right):
 
 
 def test_bounds_are_respected():
-    assert minimal(floats(min_value=1.0), lambda x: True) == 1.0
-    assert minimal(floats(max_value=-1.0), lambda x: True) == -1.0
+    assert minimal(floats(min_value=1.0)) == 1.0
+    assert minimal(floats(max_value=-1.0)) == -1.0
 
 
 @pytest.mark.parametrize("k", range(10))
 def test_floats_from_zero_have_reasonable_range(k):
-    n = 10 ** k
+    n = 10**k
     assert minimal(floats(min_value=0.0), lambda x: x >= n) == float(n)
     assert minimal(floats(max_value=0.0), lambda x: x <= -n) == float(-n)
 
@@ -211,10 +209,10 @@ class TestFloatsAreFloats:
     def test_unbounded(self, arg):
         assert isinstance(arg, float)
 
-    @given(floats(min_value=0, max_value=float(2 ** 64 - 1)))
+    @given(floats(min_value=0, max_value=float(2**64 - 1)))
     def test_int_float(self, arg):
         assert isinstance(arg, float)
 
-    @given(floats(min_value=float(0), max_value=float(2 ** 64 - 1)))
+    @given(floats(min_value=float(0), max_value=float(2**64 - 1)))
     def test_float_float(self, arg):
         assert isinstance(arg, float)
