@@ -20,7 +20,7 @@ from hypothesis.extra import pandas as pdst
 from hypothesis.extra.pandas.impl import IntegerDtype
 
 from tests.common.arguments import argument_validation_test, e
-from tests.common.debug import find_any
+from tests.common.debug import check_can_generate_examples, find_any
 from tests.common.utils import checks_deprecated_behaviour
 
 BAD_ARGS = [
@@ -115,17 +115,19 @@ def test_timestamp_as_datetime_bounds(dt):
 
 @checks_deprecated_behaviour
 def test_confusing_object_dtype_aliases():
-    pdst.series(elements=st.tuples(st.integers()), dtype=tuple).example()
+    check_can_generate_examples(
+        pdst.series(elements=st.tuples(st.integers()), dtype=tuple)
+    )
 
 
 @pytest.mark.skipif(
     not IntegerDtype, reason="Nullable types not available in this version of Pandas"
 )
 def test_pandas_nullable_types_class():
+    st = pdst.series(dtype=pd.core.arrays.integer.Int8Dtype)
     with pytest.raises(
         InvalidArgument, match="Otherwise it would be treated as dtype=object"
     ):
-        st = pdst.series(dtype=pd.core.arrays.integer.Int8Dtype)
         find_any(st, lambda s: s.isna().any())
 
 
@@ -140,4 +142,4 @@ def test_pandas_nullable_types_class():
 )
 def test_invalid_datetime_or_timedelta_dtype_raises_error(dtype_, expected_unit):
     with pytest.raises(InvalidArgument, match=re.escape(expected_unit)):
-        pdst.series(dtype=dtype_).example()
+        check_can_generate_examples(pdst.series(dtype=dtype_))
