@@ -32,12 +32,15 @@ def test_mypy_passes_on_hypothesis_strict():
 
 
 def get_mypy_output(fname, *extra_args):
-    return subprocess.run(
-        [tool_path("mypy"), *extra_args, str(fname)],
+    proc = subprocess.run(
+        [tool_path("mypy"), "--no-incremental", *extra_args, str(fname)],
         encoding="utf-8",
         capture_output=True,
         text=True,
-    ).stdout
+    )
+    if proc.stderr:
+        raise AssertionError(f"{proc.returncode=}\n\n{proc.stdout}\n\n{proc.stderr}")
+    return proc.stdout
 
 
 def get_mypy_analysed_type(fname):
@@ -89,6 +92,8 @@ def assert_mypy_errors(fname, expected, python_version=None):
     # Shell output looks like:
     # file.py:2: error: Incompatible types in assignment ... [assignment]
 
+    print(f"mypy output: {out}")
+
     def convert_lines():
         for error_line in out.splitlines():
             col, category = error_line.split(":")[-3:-1]
@@ -97,8 +102,6 @@ def assert_mypy_errors(fname, expected, python_version=None):
                 # --hide-error-context. Don't include these
                 continue
 
-            # Intentional print so we can check mypy's output if a test fails
-            print(error_line)
             error_code = error_line.split("[")[-1].rstrip("]")
             if error_code == "empty-body":
                 continue
@@ -336,14 +339,7 @@ def test_stateful_target_params_mutually_exclusive(tmp_path, decorator):
 
 @pytest.mark.parametrize("decorator", ["rule", "initialize"])
 @pytest.mark.parametrize(
-    "target_args",
-    [
-        "target=b1",
-        # FIXME: temporary workaround for mypy bug, see hypothesis/pull/4136
-        pytest.param("targets=(b1,)", marks=pytest.mark.xfail(strict=False)),
-        pytest.param("targets=(b1, b2)", marks=pytest.mark.xfail(strict=False)),
-        "",
-    ],
+    "target_args", ["", "target=b1", "targets=(b1,)", "targets=(b1, b2)"]
 )
 @pytest.mark.parametrize("returns", ["int", "MultipleResults[int]"])
 def test_stateful_target_params_return_type(tmp_path, decorator, target_args, returns):
